;
; small code stubs in RAM so NSF can play everywhere it needs to
; also allocates all RAM variables in a common place
;

.export sta_5FF8, sta_5FF9, sta_5FFA, sta_5FFB, sta_5FFC, sta_5FFD, sta_5FFE, sta_5FFF
.export sty_5FFC, sty_5FFD
.export main_loop
.export vec_nmi, vec_reset, vec_irq

.exportzp nmi_count     ; ticks by one every NMI
.exportzp gamepad       ; gamepad state
.exportzp temp_word0    ; temporary word variable
.exportzp temp_word1    ; temporary word variable
.exportzp temp_word2    ; temporary word variable
.exportzp temp_word3    ; temporary word variable
.export bank_add        ; number to add to bankswitch
.export last_8000       ; bank at 8000 when player.s not active
.export last_9000       ; bank at 9000 when player.s not active
.export pal             ; detected: 0=NTSC, 1=PAL, 2=DENDY
.export fps             ; 60 NTSC or 50 PAL/DENDY
.export frame           ; counts 0-fps
.export sec0            ; counts 0-9 as frame ticks over
.export sec1            ; counts 0-5 as sec0 ticks over
.export min0            ; counts 0-9 as sec1 ticks over
.export min1            ; counts 0-9
.export play_addr       ; NSF play address
.export init_addr       ; NSF init address
.export track_nsf       ; NSF track to play
.export pal_nsf         ; NSF pal mode
.export pal_adjust      ; counts 0-5 for doubled frames
.export pal_adjust_on   ; 0 if not doubling every 5th frame
.export main_exit       ; 1 to return to player main_exit, disable NMI first! 2 = play new track
.export play            ; play flag, 0 to pause audio
.export player_on       ; track screen running
.export play_through    ; continuous play
.export play_seconds    ; seconds of track played
.export track           ; current track
.export menu_pos        ; menu position
.export menu_hold       ; wait for input to clear
.export menu_blip       ; menu blip sound
.export gfx_pal         ; 32 byte palette
.export gfx_nmt         ; 32 byte nametable update
.export gfx_latch       ; address for nametable update
.export gfx_len         ; bytes in nametable update
.export gfx_ready       ; 1 = NMI should upload palette
.export oam_pos
.export param_a
.export param_b
.export param_c
.export param_d
.export anim_a
.export anim_b
.export anim_c
.export anim_d
.export prng_seed

.export PARTICLE_COUNT
.export particle_x
.export particle_y
.export particle_p
.export particle_s
.export particle_t
.export speed_rng
.export tick_cycle
.export slash_slider
.export quad_hack
.export cheat_code


.segment "ZEROPAGE"
nmi_count:     .res 1
gamepad:       .res 1
temp_word0:    .res 2
temp_word1:    .res 2
temp_word2:    .res 2
temp_word3:    .res 2

.segment "BSS"
bank_add:      .res 1
last_8000:     .res 1
last_9000:     .res 1
pal:           .res 1
fps:           .res 1
frame:         .res 1
sec0:          .res 1
sec1:          .res 1
min0:          .res 1
min1:          .res 1
play_addr:     .res 2
init_addr:     .res 2
track_nsf:     .res 1
pal_nsf:       .res 1
pal_adjust:    .res 1
pal_adjust_on: .res 1
main_exit:     .res 1
play:          .res 1
player_on:     .res 1
play_through:  .res 1
play_seconds:  .res 2
track:         .res 1
menu_pos:      .res 1
menu_hold:     .res 1
menu_blip:     .res 1
gfx_pal:       .res 32
gfx_nmt:       .res 32
gfx_latch:     .res 2
gfx_len:       .res 2
gfx_ready:     .res 1
oam_pos:       .res 1
param_a:       .res 1
param_b:       .res 1
param_c:       .res 1
param_d:       .res 1
anim_a:        .res 1
anim_b:        .res 1
anim_c:        .res 1
anim_d:        .res 1
prng_seed:     .res 2

.ifdef NSF
PARTICLE_COUNT = 1
.else
PARTICLE_COUNT = 16
.endif

particle_x:    .res PARTICLE_COUNT ; X coordinate
particle_y:    .res PARTICLE_COUNT ; Y coordinate
particle_p:    .res PARTICLE_COUNT ; palette OR
particle_s:    .res PARTICLE_COUNT ; speed (x/255 pixels per frame)
particle_t:    .res PARTICLE_COUNT ; type
speed_rng:     .res 1
tick_cycle:    .res 1
slash_slider:  .res 1
quad_hack:     .res 1
cheat_code:    .res 1


.segment "RAMCODE"

;
; NSF
;

.ifdef NSF

.export nsf_ramcode_init
nsf_ramcode_init:
	pha ; store track to play
	; player initialize
	lda #$00
	sta $5FF8
	lda #$01
	sta $5FF9
	pla
	sta track
	jsr $8006 ; player.s track_play
	; NSF initialize
	lda last_8000
	sta $5FF8
	lda last_9000
	sta $5FF9
	lda track_nsf
	ldx pal_nsf
	ldy #0
	jsr do_init
	rts

.export nsf_ramcode_play
nsf_ramcode_play:
	jsr do_play
	; HACK
	lda quad_hack
	beq :+
		jsr do_play
		jsr do_play
		jsr do_play
	:
	; end HACK
	lda pal_adjust_on ; pal adjust = double every 5th frame
	beq :++
		inc pal_adjust
		lda pal_adjust
		cmp #5
		bne :+
			jsr do_play
			lda #0
			sta pal_adjust
			; HACK
			lda quad_hack
			beq :+
				jsr do_play
				jsr do_play
				jsr do_play
			; end HACK
		:
	:
	rts

.endif

; replacements for STA abs bank switching

sta_5FF8:
	php
	pha
	clc
	adc bank_add
	sta last_8000 ; keep track of it for NMI handler
	sta $5FF8
	pla
	plp
	rts

sta_5FF9:
	php
	pha
	clc
	adc bank_add
	sta last_9000 ; keep track of it for NMI handler
	sta $5FF9
	pla
	plp
	rts

sta_5FFA:
	php
	pha
	clc
	adc bank_add
	sta $5FFA
	pla
	plp
	rts

sta_5FFB:
	php
	pha
	clc
	adc bank_add
	sta $5FFB
	pla
	plp
	rts

sta_5FFC:
	php
	pha
	clc
	adc bank_add
	sta $5FFC
	pla
	plp
	rts

sta_5FFD:
	php
	pha
	clc
	adc bank_add
	sta $5FFD
	pla
	plp
	rts

sta_5FFE:
	php
	pha
	clc
	adc bank_add
	sta $5FFE
	pla
	plp
	rts

sta_5FFF:
	php
	cmp #$FF
	bne :+
		sta $5FFF
		plp
		rts
	:
	pha
	clc
	adc bank_add
	sta $5FFF
	pla
	plp
	rts

; replacements for STY abs bankswitching

sty_5FFC:
	php
	pha
	tya
	clc
	adc bank_add
	sta $5FFC
	pla
	plp
	rts

sty_5FFD:
	php
	pha
	tya
	clc
	adc bank_add
	sta $5FFD
	pla
	plp
	rts

; trampolines for playback/init

do_play:
	jmp (play_addr)
	; rts provided by routine at play_addr

do_init:
	jmp (init_addr)
	; rts provided by routine at init_addr

; main playback loop

main_loop:
	lda last_8000
	sta $5FF8
	lda last_9000
	sta $5FF9
	lda track_nsf
	ldx pal_nsf
	ldy #0
	jsr do_init
	lda $2002
	lda #%10001000
	sta $2000 ; turn on NMI
	:
		lda nmi_count
		:
			cmp nmi_count
			beq :-
		;
		lda main_exit
		bne :+
		jmp :--
	:
	lda #0
	sta $2000 ; disable NMI
	sta $2001 ; disable rendering
	sta gfx_ready ; cancel any pending update
	sta gfx_len
	sta $5FF8 ; switch to player bank
	lda #1
	sta $5FF9
	jmp $8003 ; main_return in player.s

; IRQ vectors

.ifdef NSF
vec_nmi:
	rti
.else

vec_nmi:
	pha
	txa
	pha
	tya
	pha
	lda #$00
	sta $5FF8
	lda #$01
	sta $5FF9
	jsr $8000 ; $8000 = player nmi_handler
	lda play
	beq @play_end
	lda last_8000
	sta $5FF8
	lda last_9000
	sta $5FF9
	jsr do_play
	; HACK
	lda quad_hack
	beq :+
		jsr do_play
		jsr do_play
		jsr do_play
	:
	; end HACK
	lda pal_adjust_on ; pal adjust = double every 5th frame
	beq :++
		inc pal_adjust
		lda pal_adjust
		cmp #5
		bne :+
			jsr do_play
			lda #0
			sta pal_adjust
			; HACK
			lda quad_hack
			beq :+
				jsr do_play
				jsr do_play
				jsr do_play
			;
		:
	:
@play_end:
	inc nmi_count
	pla
	tay
	pla
	tax
	pla
	rti

.endif

vec_reset:
	lda #$FF
	sta $5FFF
	jmp ($FFFC)

vec_irq:
	rti

; end of file
